﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace HierarchyBindingIssue
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = new VehicleBusinessLogic();
        }

    }
    public class VehicleBusinessLogic
    {
        ObservableCollection<ManyToMany> _manytomany = null;

        public ObservableCollection<ManyToMany> ManyToMany
        {
            get
            {
                if (this._manytomany == null)
                {
                    _manytomany = new ObservableCollection<ManyToMany>();

                    ManyToMany s1 = new ManyToMany("Something", "MyCaption");
                    ManyToMany s2 = new ManyToMany("Something else", "MyCaption");

                    Car c1 = new Car("XYZ123", new DateTime(2001, 2, 10));
                    Car c2 = new Car("ZYX321", new DateTime(2002, 4, 15));
                    Car c3 = new Car("ABC987", new DateTime(2003, 6, 20));
                    Car c4 = new Car("CBA789", new DateTime(2004, 8, 25));

                    c1.AddOwner("David", "McCullough", new DateTime(2001, 2, 10), new DateTime(2004, 10, 2));
                    c1.AddOwner("Ron", "Chernow", new DateTime(2004, 10, 2), null);
                    c2.AddOwner("Thomas", "Hardy", new DateTime(2002, 4, 15), new DateTime(2006, 11, 4));
                    c2.AddOwner("Martín", "Sommer", new DateTime(2006, 11, 4), null);
                    c3.AddOwner("Victoria", "Ashworth", new DateTime(2003, 6, 20), new DateTime(2008, 3, 3));
                    c3.AddOwner("Ann", "Devon", new DateTime(2008, 3, 3), null);
                    c4.AddOwner("Maria", "Larsson", new DateTime(2004, 8, 25), new DateTime(2008, 7, 9));
                    c4.AddOwner("Carine", "Schmitt", new DateTime(2008, 7, 9), null);

                    s1.ThisCar = c1;
                    s1.ThisCar = c2;
                    s1.ThisCar = c3;
                    s1.ThisCar = c4;

                    ManyToMany.Add(s1);
                    ManyToMany.Add(s2);
                }

                return _manytomany;
            }
        }
    }

    public class ManyToMany : INotifyPropertyChanged
    {
        string m_something;
        string m_caption;
        Car m_car;
        public ManyToMany(string something, string caption)
        {
            this.Something = something;
            this.Caption = caption;
        }

        public string Something
        {
            get { return m_something; }
            set
            {
                if (m_something != value)
                {
                    m_something = value;
                    NotifyPropertyChanged("Something");
                }
            }
        }

        public string Caption
        {
            get { return m_caption; }
            set
            {
                if (m_caption != value)
                {
                    m_caption = value;
                    NotifyPropertyChanged("Caption");
                }
            }
        }

        public Car ThisCar
        {
            get { return m_car; }
            set
            {
                if (m_car != value)
                {
                    m_car = value;
                    NotifyPropertyChanged("ThisCar");
                }
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
        #endregion
    }

    public class Car : INotifyPropertyChanged
    {
        string m_registrationid;
        DateTime? m_registrationdate;
        ObservableCollection<Owner> m_owners = new ObservableCollection<Owner>();

        public Car(string registrationid, DateTime? registrationdate)
        {
            this.RegistrationId = registrationid;
            this.RegistrationDate = registrationdate;
        }

        public void AddOwner(string givenname, string surname, DateTime ownerfrom, DateTime? ownerto)
        {
            this.Owners.Add(new Owner(givenname, surname, ownerfrom, ownerto));
        }

        public string RegistrationId
        {
            get { return m_registrationid; }
            set
            {
                if (m_registrationid != value)
                {
                    m_registrationid = value;
                    NotifyPropertyChanged("RegistrationId");
                }
            }
        }

        public DateTime? RegistrationDate
        {
            get { return m_registrationdate; }
            set
            {
                if (m_registrationdate != value)
                {
                    m_registrationdate = value;
                    NotifyPropertyChanged("RegistrationDate");
                }
            }
        }

        public ObservableCollection<Owner> Owners
        {
            get { return m_owners; }
            set
            {
                if (m_owners != value)
                {
                    m_owners = value;
                    NotifyPropertyChanged("Owners");
                }
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
        #endregion
    }

    public class Owner : INotifyPropertyChanged
    {
        string m_givenname = string.Empty;
        string m_surname = string.Empty;
        DateTime m_ownerfrom = DateTime.MinValue;
        DateTime? m_ownerto = null;

        public Owner(string givenname, string surname, DateTime ownerfrom, DateTime? ownerto)
        {
            this.GivenName = givenname;
            this.SurName = surname;
            this.OwnerFrom = ownerfrom;
            this.OwnerTo = ownerto;
        }

        public string GivenName
        {
            get { return m_givenname; }
            set
            {
                if (m_givenname != value)
                {
                    m_givenname = value;
                    NotifyPropertyChanged("GivenName");
                }
            }
        }

        public string SurName
        {
            get { return m_surname; }
            set
            {
                if (m_surname != value)
                {
                    m_surname = value;
                    NotifyPropertyChanged("SurName");
                }
            }
        }

        public DateTime OwnerFrom
        {
            get { return m_ownerfrom; }
            set
            {
                if (m_ownerfrom != value)
                {
                    m_ownerfrom = value;
                    NotifyPropertyChanged("OwnerFrom");
                }
            }
        }

        public DateTime? OwnerTo
        {
            get { return m_ownerto; }
            set
            {
                if (m_ownerto != value)
                {
                    m_ownerto = value;
                    NotifyPropertyChanged("OwnerTo");
                }
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
        #endregion
    }
}
